@@ -18,9 +18,7 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -2100,18 +2098,19 @@ get_icons (WnckWindow *window)
 {
   GdkPixbuf *icon;
   GdkPixbuf *mini_icon;
+  gsize normal_size;
+  gsize mini_size;
 
   icon = NULL;
   mini_icon = NULL;
+  normal_size = _wnck_get_default_icon_size ();
+  mini_size = _wnck_get_default_mini_icon_size ();
 
   if (_wnck_read_icons (WNCK_SCREEN_XSCREEN (window->priv->screen),
                         window->priv->xwindow,
                         window->priv->icon_cache,
-                        &icon,
-                        DEFAULT_ICON_WIDTH, DEFAULT_ICON_HEIGHT,
-                        &mini_icon,
-                        DEFAULT_MINI_ICON_WIDTH,
-                        DEFAULT_MINI_ICON_HEIGHT))
+                        &icon, normal_size, normal_size,
+                        &mini_icon, mini_size, mini_size))
     {
       window->priv->need_emit_icon_changed = TRUE;
 
@@ -2129,6 +2128,18 @@ get_icons (WnckWindow *window)
             !(window->priv->icon || window->priv->mini_icon));
 }
 
+void
+_wnck_window_load_icons (WnckWindow *window)
+{
+  g_return_if_fail (WNCK_IS_WINDOW (window));
+
+  get_icons (window);
+  if (window->priv->need_emit_icon_changed)
+    queue_update (window); /* not done in get_icons since we call that from
+                            * the update
+                            */
+}
+
 /**
  * wnck_window_get_icon:
  * @window: a #WnckWindow.
@@ -2146,11 +2157,7 @@ wnck_window_get_icon (WnckWindow *window)
 {
   g_return_val_if_fail (WNCK_IS_WINDOW (window), NULL);
 
-  get_icons (window);
-  if (window->priv->need_emit_icon_changed)
-    queue_update (window); /* not done in get_icons since we call that from
-                            * the update
-                            */
+  _wnck_window_load_icons (window);
 
   return window->priv->icon;
 }
@@ -2172,11 +2179,7 @@ wnck_window_get_mini_icon (WnckWindow *window)
 {
   g_return_val_if_fail (WNCK_IS_WINDOW (window), NULL);
 
-  get_icons (window);
-  if (window->priv->need_emit_icon_changed)
-    queue_update (window); /* not done in get_icons since we call that from
-                            * the update
-                            */
+  _wnck_window_load_icons (window);
 
   return window->priv->mini_icon;
 }
@@ -2598,7 +2601,9 @@ _wnck_window_process_property_notify (WnckWindow *window,
       queue_update (window);
     }
   else if (xevent->xproperty.atom ==
-           _wnck_atom_get ("_NET_FRAME_EXTENTS"))
+           _wnck_atom_get ("_NET_FRAME_EXTENTS") ||
+           xevent->xproperty.atom ==
+           _wnck_atom_get ("_GTK_FRAME_EXTENTS"))
     {
       window->priv->need_update_frame_extents = TRUE;
       queue_update (window);
